package j51.intel;

import j51.util.Hex;

/**
 * $Id$
 * $Name$
 *
 * Abstract class to implement one MCS51 instruction. This class manage
 * all the operation relative to description get/set counter and the
 * other method of MCS51Opcode not relative to the execution of the
 * instruction using this superclass the only method that a child must
 * implement is the exec and toString used for decode the instruction.
 * 
 * $Log$
 */
public abstract class AbstractMCS51Opcode implements MCS51Opcode
{
	private int length;
	protected String description;
	protected int opcode;
	private int cycle;
	private long counter;

	AbstractMCS51Opcode(int opcode,int length,int cycle,String description)
	{
		this.opcode = opcode;
		this.length = length;
		this.cycle  = cycle;
		this.description = description;
		this.counter = 0;
	}

	public final void incCounter()
	{
		counter++;
	}

	public final void clearCounter()
	{
		counter = 0;
	}

	public final long getCounter()
	{
		return counter;
	}

	public final int getOpcode()
	{
		return opcode;
	}

	public String toString()
	{
		return description;
	}

	public final int getLength()
	{
		return length;
	}

	public final int getCycle()
	{
		return cycle;
	}

	
	public final String decode(MCS51 cpu,int pc)
	{
		String buffer = Hex.bin2word(pc)+" ";
		for (int i = 0 ; i < 3 ; i++)
		{
			if (i < getLength())
				buffer += Hex.bin2byte(cpu.code(pc+i));
			else
				buffer += "  ";
			buffer += " ";
		}

		buffer += getDescription();

		return buffer;
	}

	/**
	 * Return the description of the instruction this implementation
	 * is based over ther convention that the method toString() return
	 * the instruction description with the first word separated by tab
	 * from the operand and expand the first word plus the tab in one
	 * fixed lenght string. Example if the toString() return "CPL\tC"
	 * will be expanded in "CPL   C", to have all the instrction
	 * description with the same length.
	 */
	public final String getDescription()
	{
		String s = toString();
		int n = s.indexOf('\t');

		if (n < 0)
			return s;

		String s1 = s.substring(0,n);
		while (s1.length() < 6)
			s1 = s1+" ";
		s1 += s.substring(n+1);

		return s1;
	}

}

