package j51.intel;

import java.awt.*;
import javax.swing.*;
import java.util.Vector;

import j51.util.Hex;
import j51.swing.*;

/**
 * $Id$
 * $Name$
 *
 * 8051 standard I/O port emulator.
 *
 * $Log$
 */
public class JPort extends JPanel implements MCS51Peripheral,SfrWriteListener,MCS51Constants
{
	private int sfrPort[];
	private int sfrM1[];
	private int sfrM2[];
	private int disabled[];
	
	private JCheckBox box[][];
	private JHexByte pm[][];

	public JPort(int numPort)
	{
		super(new GridBagLayout());
		JFactory.setTitle(this,"Port 0-"+numPort);
		this.sfrPort = new int[numPort];
		this.sfrM1		= new int[numPort];
		this.sfrM2		= new int[numPort];
		this.disabled	= new int[numPort];
		
		this.box = new JCheckBox[numPort][8];
		this.pm	= new JHexByte[numPort][2];

		GridBagConstraints g = new GridBagConstraints();
		g.gridx = 0; g.gridy = 0;g.gridwidth = 1;g.gridheight = 1;
		g.anchor = g.CENTER; g.fill = g.NONE; //g.insets=new Insets(1,1,1,1);


		g.gridx = 1;
		for (int i = 0; i < 8 ; i++)
		{
			add(new JLabel(""+(7-i)),g);
			g.gridx++;
			
		}
		
		add(new JLabel("M1"),g);
		g.gridx++;
		add(new JLabel("M2"),g);

		g.gridy++;

		for (int port = 0 ; port < sfrPort.length ; port++)
		{
			disabled[port] = 0;
			pm[port][0] = new JHexByte();
			pm[port][1] = new JHexByte();

			switch (port)
			{
				case	0:
						sfrPort[port]	= P0;
						sfrM1[port]		= P0M1;
						sfrM2[port]		= P0M2;
						break;
				case	1:
						sfrPort[port]	= P1;
						sfrM1[port]		= P1M1;
						sfrM2[port]		= P1M2;
						break;
				case	2:
						sfrPort[port] = P2;
						sfrM1[port]		= P2M1;
						sfrM2[port]		= P2M2;
						break;
				case	3:
						sfrPort[port]	= P3;
						sfrM1[port]		= P3M1;
						sfrM2[port]		= P3M2;
						break;
			}

			g.gridx = 0;
			add(new JLabel("P"+port),g);
			g.gridx++;
			for (int i = 0 ; i < 8 ; i++)
			{
				box[port][7-i] = new JCheckBox();
				add(box[port][7-i],g);
				g.gridx++;
				
			}
			
			add(pm[port][0],g);
			g.gridx++;
			add(pm[port][1],g);

			g.gridy++;

			
		}


	}

	public void registerCpu(MCS51 cpu)
	{
		for (int i = 0 ; i < sfrPort.length ; i++)
		{
			cpu.addSfrWriteListener(sfrPort[i],this);
			cpu.addSfrWriteListener(sfrM1[i],this);
			cpu.addSfrWriteListener(sfrM2[i],this);
		}
	}

	public void sfrWrite(int reg,int value)
	{
		for (int i = 0 ; i < sfrPort.length ; i++)
		{
			if (reg == sfrPort[i])
			{
				update(i,value);
				break;
			}

			if (reg == sfrM1[i])
			{
				M1(i,value);
				break;
			}

			if (reg == sfrM2[i])
			{
				M2(i,value);
				break;
			}

		}
	}

	void mode(int port)
	{
		for (int i = 0 ; i < 8 ; i++)
		{
			int mask = 1 << i;

			boolean  m1 = false , m2 = false;
			boolean dis = (disabled[port] & mask) != 0;
			
			try
			{

				m1 = (pm[port][0].getValue() & mask) != 0;
				m2 = (pm[port][1].getValue() & mask) != 0;
			}
			catch (Exception e)
			{
			}

			if (dis)
				box[port][i].setEnabled(false);
			else
			{
				if (!m1 && m2)
					box[port][i].setEnabled(false);
				else
					box[port][i].setEnabled(true);
			}
		}
	}

	public void setDisableMask(int port,int mask)
	{
		disabled[port] = mask;
	}
	
	void M1(int port,int value)
	{
		pm[port][0].setValue(value);
		mode(port);
	}

	void M2(int port,int value)
	{
		pm[port][1].setValue(value);
		mode(port);
	}

	void update(int port,int value)
	{
		for (int i = 0 ; i < 8 ; i++)
		{
			boolean mode = (value & (1 << i)) != 0;
			if (box[port][i].isSelected() != mode)
				box[port][i].setSelected(mode);
		}
	}
}


